#![cfg_attr(
    all(not(debug_assertions), target_os = "windows"),
    windows_subsystem = "windows"
)]

mod core;
mod picker;
mod rpc;

use paris::*;
use picker::led_color::LedColor;
use picker::manager::Picker;
use crate::core::CoreManager;
use std::time::Instant;

#[tauri::command]
async fn refresh_displays() {
    match Picker::global().refresh_displays().await {
        Ok(_) => {}
        Err(error) => {
            error!("{}", error)
        }
    }
}

#[tauri::command]
async fn take_snapshot() -> Vec<String> {
    let start = Instant::now();
    let manager = Picker::global();

    match manager.take_screenshots_for_all().await {
        Ok(screenshots) => {
            info!("screenshots len: {}", screenshots.len());
            let mut futures = Vec::new();
            for screenshot in screenshots {
                let future = screenshot.to_webp_base64().await;
                futures.push(future);
            }
            futures
        }
        Err(error) => {
            error!("can not take screenshots for all. {}", error);
            vec![]
        }
    }
}

#[tauri::command]
async fn get_led_strip_colors() -> Result<Vec<LedColor>, String> {
    let colors = Picker::global().get_led_strip_colors().await;
    match colors {
        Ok(colors) => {
            rpc::manager::Manager::global()
                .publish_led_colors(&colors.to_vec())
                .await;
            Ok(colors)
        }
        Err(error) => Err(format!("{}", error)),
    }
}

#[tauri::command]
async fn play_flowing_light() {
    CoreManager::global().play_flowing_light().await;
}
#[tauri::command]
async fn stop_flowing_light() {
    CoreManager::global().stop_flowing_light().await;
}

#[tokio::main]
async fn main() {
    rpc::manager::Manager::global();
    tauri::Builder::default()
        .invoke_handler(tauri::generate_handler![
            take_snapshot,
            refresh_displays,
            get_led_strip_colors,
            play_flowing_light,
            stop_flowing_light,
        ])
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
